<?php


namespace Shopbydedi\Payment;

use HiPay\Fullservice\Gateway\Client\GatewayClient;
use HiPay\Fullservice\Gateway\Request\Info\CustomerBillingInfoRequest;
use HiPay\Fullservice\Gateway\Request\Info\CustomerShippingInfoRequest;
use HiPay\Fullservice\Gateway\Request\Order\HostedPaymentPageRequest;
use HiPay\Fullservice\HTTP\Configuration\Configuration;
use HiPay\Fullservice\HTTP\SimpleHTTPClient;
use Panier;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Tpe;

/**
 * Class Hipay
 * @package Shopbydedi\Payment
 */
class Hipay implements PaymentInterface
{
    protected $urlSuccess;
    protected $urlFailure;
    protected $urlNotify;

    protected $secret;

    /**
     * @var \HiPay\Fullservice\HTTP\SimpleHTTPClient
     */
    protected $clientProvider;
    /**
     * @var \HiPay\Fullservice\Gateway\Client\GatewayClient
     */
    protected $gatewayClient;
    /**
     * @var \HiPay\Fullservice\HTTP\Configuration\Configuration
     */
    protected $config;

    /**
     * @param $config
     *
     * @throws \DediException
     */
    public function __construct($config)
    {

        $this->isConfigValid($config);

        $this->config = new Configuration($config["hipay"]["identifier"],$config["hipay"]["password"], $config["hipay"]["mode"]);

        //Instantiate client provider with configuration object
        $this->clientProvider = new SimpleHTTPClient($this->config);

        //Create your gateway client
        $this->gatewayClient = new GatewayClient($this->clientProvider);

        $this->secret = $config["hipay"]["secret"];
    }


    /**
     * @param Tpe $tpe
     * @param float $amount
     * @param array $params
     */
    public function request(Tpe $tpe, $amount, $params = array())
    {

        // TODO mettre en variable récupéré la langue actuelle
        $lang = 'fr_FR';
        // TODO mettre en variable récupéré la devise actuelle
        $currency = "EUR";

        $pageRequest = new HostedPaymentPageRequest();
        $pageRequest->orderid = $tpe->tpe_reference_id;
        $pageRequest->operation = "Sale";
        $pageRequest->description = 'Transaction numero ' . $tpe->tpe_reference_id;
        $pageRequest->amount = round($amount, 2);
        $pageRequest->currency = $currency;
        $pageRequest->accept_url = $this->urlSuccess;
        $pageRequest->pending_url = $this->urlSuccess;
        $pageRequest->decline_url = $this->urlFailure;
        // $pageRequest->exception_url = $this->urlFailure;
        $pageRequest->cancel_url = $this->urlFailure;
        $pageRequest->notify_url = $this->urlNotify;
        $pageRequest->cid = $params['client_id'];
        $pageRequest->language = $lang;
        $pageRequest->ipaddr = $_SERVER['REMOTE_ADDR'];
        $pageRequest->template = 'basic-js';
        if (!empty($params['customerBillingInfo'])) {
            $pageRequest->customerBillingInfo = $params['customerBillingInfo'];
        }
        if (!empty($params['customerShippingInfo'])) {
            $pageRequest->customerShippingInfo = $params['customerShippingInfo'];
        }

        if (!empty($params['tax'])) {
            $pageRequest->tax = $params['tax'];
        }

        if (!empty($params['shipping'])) {
            $pageRequest->shipping = $params['shipping'];
        }

        $pageRequest->payment_product_category_list = 'credit-card';

        try {
            $transaction = $this->gatewayClient->requestHostedPaymentPage($pageRequest);

            $url = $transaction->getForwardUrl();
            (new RedirectResponse($url))->send();
            exit;
        } catch (\Exception $e) {

            // TODO : logger erreur + flashbag ?
            /*dump($e->getMessage());
            exit;*/

            (new RedirectResponse($this->urlFailure))->send();
            exit;
        }

    }

    /**
     * @param Panier $panier
     * @param float $amount
     * @param array $params
     * @return mixed|void
     * @throws \Exception
     */
    public function requestFromPanier(Panier $panier, $amount, $params = array())
    {
        $client = $panier->getCustomer();
        $params['client_email'] = $client->client_email;
        $params['client_id'] = $client->client_id;

        $addrBilling = $panier->getAddressBilling();

        $customerBillingInfo = new CustomerBillingInfoRequest();
        $customerBillingInfo->email = $client->client_email;
        $customerBillingInfo->firstname = $addrBilling->adresse_prenom;
        $customerBillingInfo->lastname = $addrBilling->adresse_nom;
        $customerBillingInfo->streetaddress = $addrBilling->adresse_rue;
        $customerBillingInfo->streetaddress2 = $addrBilling->adresse_rue2;
        $customerBillingInfo->city = $addrBilling->adresse_ville;
        $customerBillingInfo->zipcode = $addrBilling->adresse_cp;
        $customerBillingInfo->country = $addrBilling->pays_iso_code_2;

        $params['customerBillingInfo'] = $customerBillingInfo;

        $addrShipping = $panier->getAddressShipping();
        $customerShippingInfo = new CustomerShippingInfoRequest();
        $customerShippingInfo->shipto_firstname = $addrShipping->adresse_prenom;
        $customerShippingInfo->shipto_lastname = $addrShipping->adresse_nom;
        $customerShippingInfo->shipto_streetaddress = $addrShipping->adresse_rue;
        $customerShippingInfo->shipto_streetaddress2 = $addrShipping->adresse_rue2;
        $customerShippingInfo->shipto_city = $addrShipping->adresse_ville;
        $customerShippingInfo->shipto_zipcode = $addrShipping->adresse_cp;
        $customerShippingInfo->shipto_country = $addrShipping->pays_iso_code_2;

        $params['customerShippingInfo'] = $customerShippingInfo;

        $shipping = $panier->getShipping();
        $params['shipping'] = (float) $shipping->port_ttc;

        list($aPayerHT, $aPayerTTC) = $panier->getAPayer();
        $params['tax'] = round($aPayerTTC - $aPayerHT, 2);

        $this->request(new Tpe($panier->getReference()), $amount, $params);
    }

    /**
     * @param array $request
     * @return mixed|void
     * @throws \DediException
     */
    public function response($request)
    {

        // on ne traite que le cas de la notification "Captured"
        if ($request['status'] != 118) {
            echo 'no message captured';
            exit;
        }

        // TODO : mettre un logger en place
        // file_put_contents(STORAGE_DIR . 'hipay_'.date('YmdHis').'.txt', print_r($request, true));

        if (empty($request['order']['id'])) {
            echo 'no data';
            exit;
        }

        $tpe = new Tpe();

        if ($request['state'] != 'completed') {
            $tpe->paiementKO($request['order']['id'], $request);

            echo sprintf('KO state : %s', $request['state']);
            exit;
        }

        $string2compute = '';

        if (isset($request['hash'])) { // If is a redirection URL
            $signature = $request['hash'];
            $parameters = $request;
            unset($parameters['hash']);
            ksort($parameters);
            foreach ($parameters as $name => $value) {
                /*if($name == 'transaction_reference') {
                    $transaction_id = $value;
                }*/
                if (!empty($value)) {
                    $string2compute .= $name . $value . $this->secret;
                }
            }
        } else { // If is a Notification
            $signature = $_SERVER['HTTP_X_ALLOPASS_SIGNATURE'];

            $postData = file_get_contents('php://input');
            $string2compute = $postData . $this->secret;
        }

        $computedSignature = sha1($string2compute);

        // TODO : mettre un logger en place
        /*$dataSignature = array(
            'signature' => $signature,
            'string2compute' => $string2compute,
            'computedSignature' => $computedSignature,
        );
        file_put_contents(STORAGE_DIR . 'hipay_return_'.date('YmdHis').'.txt', print_r($dataSignature, true));*/

        if ($computedSignature == $signature) {
            $tpe->paiementOK($request['order']['id'], $request);
            echo 'OK';
            exit;
        } else {
            $tpe->paiementKO($request['order']['id'], $request);
            echo sprintf('KO signature : %s / %s', $computedSignature, $signature);
            exit;
        }
    }

    /**
     * @param mixed $urlNotify
     */
    public function setUrlNotify($urlNotify)
    {
        $this->urlNotify = $urlNotify;
    }

    /**
     * @param mixed $urlFailure
     */
    public function setUrlFailure($urlFailure)
    {
        $this->urlFailure = $urlFailure;
    }

    /**
     * @param mixed $urlSuccess
     */
    public function setUrlSuccess($urlSuccess)
    {
        $this->urlSuccess = $urlSuccess;
    }

    /**
     * @param $config
     */
    public function isConfigValid($config)
    {
        if (empty($config["hipay"])) {
            trigger_error("Hipay n'est pas initialisé, créer les clé hipay.identifier, hipay.password, hipay.mode", E_USER_WARNING);
        }

        if (!empty($config["hipay"]) && empty($config["hipay"]["identifier"])) {
            trigger_error("Hipay il manque la clé hipay.identifier", E_USER_WARNING);
        }

        if (!empty($config["hipay"]) && empty($config["hipay"]["password"])) {
            trigger_error("Hipay il manque la clé hipay.password", E_USER_WARNING);
        }

        if (!empty($config["hipay"]) && empty($config["hipay"]["mode"])) {
            trigger_error("Hipay il manque la clé hipay.mode", E_USER_WARNING);
        }
    }

}